@@ -4,9 +4,9 @@
  */
 
 #include "blowfish.h"
-#ifndef SWAP_H
-# define SWAP_H
-# include <swap.h>
+#ifndef XMEMORY_H
+# define XMEMORY_H
+# include <xmemory.h>
 #endif
 
 
