@@ -16,7 +16,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#ifndef LIBDEFS_H
+# define LIBDEFS_H
+# include <libdefs.h>
+#endif
 
 void Bzero( void *s, int n) {
 #ifdef HAVE_MEMSET
