@@ -204,26 +204,25 @@ int strip8bit(char * chain,unsigned int sizeofchain)
   char tmpchain[128];
   int i,j,x,y;
 
-Bzero(tmpchain,sizeof(tmpchain));
+
+  Bzero(tmpchain,sizeof(tmpchain));
     
   y=0;
+  x=1;
+  
   for (i=0;i<sizeofchain;i++) {
-
-    x=1+i;
-    
     for (j=0;j<8;j++) {
-      tmpchain[i]=setbit(tmpchain[i], j, getbit(chain[y],x) );
-
+      tmpchain[i]=setbit(tmpchain[i], j, getbit(chain[y],x));
       if (x==7) { x=0; y++; }
       x++;
     }
-  
-    if (y >= sizeofchain) break;
+                /* I think this is not needed but... */
+    if (y >= sizeofchain) break; 
   }
 
 
 Bzero(chain,sizeofchain);
-strncat(chain,tmpchain,i);
+strncpy(chain,tmpchain,i);
 
 return i;
 
