@@ -21,6 +21,8 @@
 #endif
 #include <hex.h>
 #include <ctype.h>
+#include <extra.h>
+#include <xmemory.h>
 
 int hexToChar (void *given_chain, int len) {
 
@@ -34,7 +36,7 @@ int hexToChar (void *given_chain, int len) {
  if (len % 2 !=0) return -1;
  if (check_hex(chain, len)==-1) return -1;
 
- tmpchain=malloc((len/2)+1);
+ tmpchain=mxmalloc((len/2)+1);
  
  for (i=0; i<len; i+=2) {
    strncpy( tmp, &chain[i], 2);
