@@ -23,7 +23,7 @@
 #include <hex.h>
 #include <bzero.h>
 #include <bits.h>
-
+#include <xmemory.h>
 void* fixkey(char * key, unsigned int* len, int bit7_flag, int hexflag,int algorithm, int quiet, int stream_flag, int enc_mode) {
 
 char *tmp;
@@ -39,13 +39,13 @@ int keysize;
   }
  
   if (len!=0) {
-    tmpkey=malloc(*len);
+    tmpkey=mxmalloc(*len);
   }
   else {
-    tmpkey=malloc(128);
+    tmpkey=mxmalloc(128);
   }
 
-  keyword=calloc(1, get_key_size(algorithm));
+  keyword=mxcalloc(1, get_key_size(algorithm));
 
 
   if (key==NULL) 	{
