@@ -24,7 +24,7 @@
 */
 
 #include <mcrypt.h>
-#include <swap.h>
+#include <xmemory.h>
 #include <bzero.h>
 #include <extra.h>
 #include <bits.h>
@@ -38,7 +38,9 @@
 #include <random.h>
 #include <gaa.h>
 #include <errors.h>
+#include <environ.h>
 
+ char tmperr[128];
  unsigned int bit7_flag, hexflag=FALSE;
  unsigned int stream_flag=FALSE;
  unsigned int mode=CBC; /* 0:cbc 1:ecb */
@@ -52,7 +54,7 @@
 void usage(char *name)
 {
 
-     fprintf(stderr,_("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k keyword] [-m mode] [-i bitmode] [-a algorithm] [-c config_file] [file ...]\n\n"));
+     fprintf(stderr,_("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k key1 key2 ...] [-m mode] [-i bitmode] [-a algorithm] [-c config_file] [file ...]\n\n"));
 
      fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
      fprintf(stderr,_("Legal parameters are:\n\
@@ -70,7 +72,7 @@ void usage(char *name)
  -k --key KEY1, KEY2, ...     keyword(s) to use.\n\
  -f --keyfile FILENAME        file to read the keyword from.\n\
  -x --hex                     keyword in hex.\n\
- -i --bit MODE                keyword bit mode. Can be 7 or 8bit.\n\
+ -i --bit MODE                keyword bit mode. Can be 7 or 8.\n\
  -u --unlink                  unlink the input file after encryption/decryption\n\
  -s --sunlink                 unlink and destroy the input file after\n\
                               encryption/decryption\n\
@@ -115,13 +117,14 @@ void mcrypt_version() {
 /*				The main    */
 int main(int argc,char **argv)
 {
-        char tmperr[128];
+
 	short int ein=0,din=0,kin=0,force=0,return_val=0;
 	char *einfile=0 ,*dinfile=0, *keyword=0,*modet=0,*typet=0;
 	char **file, **keyfile=NULL, *cfile;
 	int x,y,i,file_count=0, keys=0, used_algo=FALSE, v;
         gaainfo info;
 
+
 #ifdef HAVE_SIGNAL_H
  Signal( SIGINT , shandler);
  Signal( SIGQUIT, shandler);
@@ -141,25 +144,13 @@ int main(int argc,char **argv)
 #endif
 
 
-/* Examine how we were called */
-   if (strstr(argv[0],"decrypt")!=NULL) {
-     strcpy( argv[0], "mdecrypt");
-     din=TRUE;
-     ein=FALSE;
-   }
-   else {
-     strcpy( argv[0], "mcrypt");
-     din=FALSE;
-     ein=TRUE; /* It will change by the arguments */
-   }
 
         if((v = gaa(argc, argv, &info)) != -1) {
-          err_quit(_("Error in arguments.\n"));
+          err_quit(_("Error in the arguments.\n"));
         }
-
-
+        
           if (info.config_file!=NULL) {
-            cfile=malloc(strlen(info.config_file));
+            cfile=mxmalloc(strlen(info.config_file));
             strcpy(cfile,info.config_file);
             if (check_file(cfile)!=1) {
               err_crit (_("Warning: Cannot access config file.\n"));
@@ -169,7 +160,7 @@ int main(int argc,char **argv)
 #ifdef HAVE_GETPWUID          
             cfile=get_cfile(getuid(), ".mcryptrc");
 #else
-            cfile=malloc(11);
+            cfile=mxmalloc(11);
             strcpy(cfile,".mcryptrc");
 #endif          
           }
@@ -184,15 +175,58 @@ int main(int argc,char **argv)
          
          free(cfile);
 
-        
 
+ if (check_env()==TRUE) {
 
+  if (get_env_key()!=NULL) { 
+   info.keysize=1;
+   free(info.keys);
+   info.keys=get_env_key();
+  }
+
+  if (get_env_algo()!=NULL) { 
+   free(info.algorithm);
+   info.algorithm=get_env_algo();
+  }
+
+  if (get_env_mode()!=NULL) { 
+   free(info.mode);
+   info.mode=get_env_mode();
+  }
+  
+  if (get_env_bit_mode()!=0) { 
+   info.bit=get_env_bit_mode();
+  }
+ }
+
+/* Check again the command line variables 
+ * This will be uncommented when gaa is corrected.
+ */
+/*
+        if((v = gaa(argc, argv, &info)) != -1) {
+          err_quit(_("Error in the arguments.\n"));
+        }
+*/
+
+/* Examine how we were called */
+   if (strstr(argv[0],"decrypt")!=NULL) {
+     Bzero(argv[0], strlen(argv[0]));
+     strcpy( argv[0], "mdecrypt");
+     din=TRUE;
+     ein=FALSE;
+   }
+   else {
+     Bzero(argv[0], strlen(argv[0]));
+     strcpy( argv[0], "mcrypt");
+     din=FALSE;
+     ein=TRUE; /* It will change by the arguments */
+   }
 
 
 
             /* File pointers are as much as the file arguments */
 
-        file=malloc((info.size)*sizeof(char*)); 
+        file=mxmalloc((info.size)*sizeof(char*)); 
     
         if (info.ed_specified!=0) {
           din=info.din;
@@ -203,7 +237,7 @@ int main(int argc,char **argv)
 #ifdef HAVE_DEV_RANDOM
           real_random_flag=TRUE;
 #else
-          err_warn (_("This system does not support real random data.\n"));
+          err_warn (_("Warning: This system does not support real random data.\n"));
 #endif
         }
 
@@ -220,11 +254,17 @@ int main(int argc,char **argv)
                   bit7_flag=BITS7;
                 }
                 else {
-                  bit7_flag=BITS8; /* default */
+                  if (info.bit==8) {
+                    bit7_flag=BITS8; /* default */
+                  }
+                  else {
+                    err_crit(_("Only 7 and 8 bit mode are supported. Assuming 8 bit mode.\n"));
+                    bit7_flag=BITS8; /* default */
+                  }
                 }
               
 
-                if (info.keyfile!=NULL) {
+               if (info.keyfile!=NULL) {
                   kin=2;
                   keyfile = read_key_file(info.keyfile, &keys);
                   if (keyfile==NULL) kin=0;
@@ -233,18 +273,18 @@ int main(int argc,char **argv)
                else {
                 if (info.keys!=NULL) {
                  kin=2;
-                 keyfile=malloc(info.keysize*sizeof(char*));
+                 keyfile=mxmalloc(info.keysize * sizeof(char*));
                  keys=info.keysize;
                 
                  for (i=0; i<info.keysize; i++) {
-                   keyfile[i]=malloc(strlen(info.keys[i])+10);
+                   keyfile[i]=mxmalloc(strlen(info.keys[i])+10);
                    strcpy(keyfile[i],info.keys[i]);
                  }
                 }
                }
 
               if (info.mode!=NULL) {
-		modet= malloc(strlen(info.mode)+1);
+		modet= mxmalloc(strlen(info.mode)+1);
 		strcpy(modet,info.mode);
 			if (strncasecmp(modet,"ECB",3)==0) {
 			  mode=ECB;
@@ -272,7 +312,7 @@ int main(int argc,char **argv)
                     
                     if (info.algorithm!=NULL) {
                         used_algo=TRUE;
-			typet= malloc(strlen(info.algorithm)+1);
+			typet= mxmalloc(strlen(info.algorithm)+1);
 			strcpy(typet,info.algorithm);
 			if (strncasecmp(typet,"DES",3)==0) {
 			  type=DES;
@@ -337,6 +377,10 @@ int main(int argc,char **argv)
                       } /* if */ 
 
 
+        if (info.keysize!=0 && check_env==FALSE) 
+           err_warn(_("Warning: It is insecure to specify keywords in the command line\n"));
+
+
 #ifdef HAVE_UMASK
   umask(066);
 #endif
@@ -420,7 +464,7 @@ for(i=0;i<file_count;i++) {
              
             if (dinfile[y-1]=='c' && dinfile[y-2]=='n' && dinfile[y-3]=='e' && dinfile[y-4]=='.') {
 
-	      outfile= calloc(y-3,1);
+	      outfile= mxcalloc(y-3,1);
   	      strncat(outfile,dinfile,y-4);
 #ifdef HAVE_STAT
               /* But if it exists exit */
@@ -433,13 +477,13 @@ for(i=0;i<file_count;i++) {
             }
             else { /* append .dec */
 
-	     outfile=calloc(y+5,1);
+	     outfile=mxcalloc(y+5,1);
              strncpy(outfile,dinfile,y);
              strcat(outfile,".dec");
             }
          }
 	 else { /* encryption- append .enc */
-	    outfile= calloc(strlen(einfile)+5,1);
+	    outfile= mxcalloc(strlen(einfile)+5,1);
 	    strcpy(outfile,einfile);
             /* if file has already the .enc ignore it */
             if (strstr( outfile, ".enc")!=NULL) {
@@ -462,7 +506,7 @@ for(i=0;i<file_count;i++) {
        }
        else { /* if streams */
 
-         outfile=malloc(7);
+         outfile=mxmalloc(7);
          strcpy(outfile,"stdout");
 
        }
@@ -528,10 +572,12 @@ for(i=0;i<file_count;i++) {
 
 			if (x==0) {
         	            if (stream_flag==FALSE) {
-        	              if (quiet==FALSE) fprintf (stderr,_("File %s was decrypted.\n"),dinfile);
+                              sprintf(tmperr, _("File %s was decrypted\n"),dinfile);
+                              err_warn (tmperr);
         	            }
         	            else {
-        	              if (quiet==FALSE) fprintf (stderr,_("Stdin was decrypted.\n"));
+                              sprintf(tmperr, _("Stdin was decrypted\n"));
+                              err_warn (tmperr);
         	            }
 #ifdef HAVE_STAT        	           
 # ifdef HAVE_UTIME_H
@@ -597,10 +643,12 @@ for(i=0;i<file_count;i++) {
 
 	        	if (x==0) {
                           if (stream_flag==FALSE) {
-	        	   if (quiet==FALSE) fprintf (stderr,_("File %s was successfully encrypted.\n"),einfile);
+                           sprintf(tmperr, _("File %s was encrypted\n"),einfile);
+                           err_warn (tmperr);
 	        	  }
                           else {
-	        	   if (quiet==FALSE) fprintf (stderr,_("Stdin was successfully encrypted.\n"));
+                           sprintf(tmperr, _("Stdin was encrypted\n"));
+                           err_warn (tmperr);
 	        	  }
 #ifdef HAVE_STAT
 # ifdef HAVE_UTIME_H
@@ -628,7 +676,12 @@ for(i=0;i<file_count;i++) {
 
    } /* the main loop */
 
-return (return_val);
+/* Clear ALL arguments in the command line */
+  for (i=0; i<argc; i++) {
+    Bzero(argv[i],strlen(argv[i]));
+  }
+
+  return (return_val);
 
 }
 
@@ -674,7 +727,10 @@ else {
          Bzero(keyword,sizeof(keyword));
 
 	if (key==NULL) 	{
-          if (quiet<=1 && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
+          if (stream_flag==FALSE) {
+            sprintf(tmperr, _("File: %s\n"),fromfile);
+            err_warn (tmperr);
+          }
           tmp=get_password(8, bit7_flag, DECRYPT, type, &len);
           if (tmp==NULL) return 1;
           memmove(keyword, tmp, len);
@@ -765,7 +821,7 @@ int blocksize=get_block_size(algorithm);
 int td;
 
 word32 *IV=0;
-word32 *ciphertext=malloc(blocksize);
+word32 *ciphertext=mxmalloc(blocksize);
 FILE * TOF; 
 
  TOF=tmpfile();
@@ -773,7 +829,7 @@ FILE * TOF;
         
 
 if (mode==CBC || mode==CFB) {
- IV=malloc(blocksize);
+ IV=mxmalloc(blocksize);
  /* Initialisation Vector */
 
 
@@ -814,7 +870,10 @@ if (mode==CBC || mode==CFB) {
   }  
 
   if (key==NULL) {
-    if (quiet<=TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
+    if (stream_flag==FALSE) {
+      sprintf(tmperr, _("File: %s\n"),fromfile);
+      err_warn (tmperr);
+    }
   }
 
 
@@ -1002,7 +1061,7 @@ switch (mode) {
 /* Copy the temporary to RTOF */
 
  fseek(TOF,0,SEEK_SET);
- copychar=malloc(4096);
+ copychar=mxmalloc(4096);
 
  do {
     how=fread(copychar, 1, 4096, TOF);
@@ -1068,10 +1127,10 @@ int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
 int td;
 
-word32 *ciphertext_old=malloc(blocksize);
-word32 *buffer=malloc(blocksize);
-word32 *ciphertext=malloc(blocksize);
-word32 *ciphertext_orig=malloc(blocksize);
+word32 *ciphertext_old=mxmalloc(blocksize);
+word32 *buffer=mxmalloc(blocksize);
+word32 *ciphertext=mxmalloc(blocksize);
+word32 *ciphertext_orig=mxmalloc(blocksize);
 word32 *keyword;
 
 unsigned int len=0;
@@ -1102,7 +1161,10 @@ else {
 
 /* Get key */
   if (key==NULL) {
-    if (quiet<=TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
+    if (quiet<=TRUE && stream_flag==FALSE) {
+     sprintf(tmperr, _("File: %s\n"),fromfile);
+     err_warn (tmperr);
+    }
   }
 
   if (key==NULL) {
